from typing import List
import requests
from datetime import datetime
from collections import Counter
from operator import itemgetter
import time
import json
from urllib import parse

# variables
TOKEN = "BBFF-4UTuHp4GWKIcAWzQvRHMQcWZ7E4Pxw"
BASE_URL = "https://industrial.api.ubidots.com"
DEVICE_LABEL = None
VAR_LABEL = "rfid"


# function to create request
def create_request(url, headers, attemps, request_type, data=None, params=None):
    request_func = getattr(requests, request_type)
    kwargs = {"url": url, "headers": headers}
    if request_type == "post" or request_type == "patch":
        kwargs["json"] = data
    if params:
        kwargs["params"] = params
    try:
        req = request_func(**kwargs)
        status_code = req.status_code
        time.sleep(1)
        while status_code >= 400 and attemps < 5:
            req = request_func(**kwargs)
            status_code = req.status_code
            attemps += 1
            time.sleep(1)
        return req
    except Exception as e:
        print("[ERROR] There was an error with the request, details:")
        print(e)
        return None


# função para atualizar dispositivo
def update_device(device, payload, token):
    """
    Atualizar um dispositivo com a carga útil(payload) especificada
    """
    url = "{}/api/v1.6/devices/{}".format(BASE_URL, device)
    headers = {"X-Auth-Token": token, "Content-Type": "application/json"}
    req = create_request(url, headers, attemps=5, request_type="post", data=payload)
    return req


# Get values from variable
def get_values(device_label, var_label, init_ts, end_ts, token):
    result = None
    data = list()
    endpoint = "/api/v1.6/devices/{}/{}/values".format(device_label, var_label)
    params = parse.urlencode({"page_size": 50000, "start": init_ts, "end": end_ts + 86400000})
    url = "{}{}/?{}".format(BASE_URL, endpoint, params)
    headers = {"Content-Type": "application/json", "X-Auth-Token": token}
    req = create_request(url, headers, 5, "get")
    while True:
        if req is None or req.status_code >= 400:
            result = None
            break
        res = req.json()
        data.extend(res["results"])
        url = res["next"]
        if url is None:
            result = data
            break
        req = create_request(url, headers, 5, "get")
    return result


# function to convert date
def conversor_date(timestamps):
    date: List[datetime] = []
    for ts in timestamps:
        obj_date = datetime.fromtimestamp(ts / 1000)
        date.append(obj_date)
    return date


# function to count cards
def count_card(lista):
    count = dict(Counter(lista))
    code = sorted(count.items(), key=itemgetter(1), reverse=True)
    new_code = []
    soma = 0
    for cd in code:
        lista = list(cd)
        lista[1] = int(lista[1] / 2)
        cd = tuple(lista)
        new_code.append(cd)
        soma += cd[1]
    new_code.append(("all", soma))

    return new_code


# function for card difference time
def card_difference_time(results):
    diff = []
    while True:
        for first in range(1):
            for second in range(1, 2):
                if results[first]["context"] == results[second]["context"]:
                    diferenca = results[second]["date"] - results[first]["date"]
                    codigo = results[first]["context"]["cod_maq"]
                    dictionary = {codigo: str(diferenca)}
                    diff.append(dictionary)
                    results.pop(first)
                    results.pop(first)
                    break
                else:
                    results.pop(first)

        if len(results) < 2:
            break
    return diff


# Main function
def main(args):
    print(args)
    device_label = args.get("device-label")
    if device_label is None:
        return {"Performance": "Error, no device label provided..."}

    start_date = args["start-date"] * 1000
    end_date = args["end-date"] * 1000
    results = get_values(DEVICE_LABEL, VAR_LABEL, start_date, end_date, TOKEN)

    # declarando variáveis
    timestamp = [ts["timestamp"] for ts in results]
    context_num = [cont["context"]["cod_maq"] for cont in results]
    dates = conversor_date(timestamp)

    # colocando date dentro de results e excluindo create_at
    cont = 0
    for c in results:
        c.pop("created_at")
        c["date"] = dates[cont]
        cont += 1

    # invertendo a variável result
    result = results[::-1]

    # chamando a função para contagem dos cartões
    cod = count_card(context_num)

    # chamando a função para ver a diferença dos cartoes
    print(card_difference_time(result))
    input("Pressione <enter> para continuar")

    return cod
